@@ -1,4 +1,3 @@
-# $Id: selex.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::AlignIO::selex
 
@@ -56,7 +55,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution. Bug reports can be submitted via the
 web:
 
-  http://bugzilla.open-bio.org/
+  https://github.com/bioperl/bioperl-live/issues
 
 =head1 AUTHORS - Peter Schattner
 
@@ -138,6 +137,7 @@ sub next_aln {
 	     '-end'              => $end,
 	     '-description'      => $desc{$name},
 	     '-accession_number' => $accession{$name},
+ 	     '-alphabet'         => $self->alphabet,
 	     );
 
 	$aln->add_seq($seq);
