@@ -121,6 +121,10 @@ additional arguments are as follows:
  -autoindex        Boolean flag. If true, features in the database will be
                    reindexed every time they change. This is the default.
 
+ -fts              Boolean flag. If true, when the -create flag is true, the
+                   attribute table will be created and indexed index for
+                   full-text search using the most recent FTS extension
+                   supported by DBD::SQLite.
 
  -tmpdir           Directory in which to place temporary files during "fast" loading.
                    Defaults to File::Spec->tmpdir(). (synonyms -dump_dir, -dumpdir, -tmp)
@@ -144,9 +148,10 @@ use strict;
 
 use base 'Bio::DB::SeqFeature::Store::DBI::mysql';
 use Bio::DB::SeqFeature::Store::DBI::Iterator;
+use DBD::SQLite;
 use DBI qw(:sql_types);
 use Memoize;
-use Cwd 'abs_path';
+use Cwd qw(abs_path getcwd);
 use Bio::DB::GFF::Util::Rearrange 'rearrange';
 use Bio::SeqFeature::Lite;
 use File::Spec;
@@ -228,6 +233,7 @@ sub init {
       $pass,
       $dbi_options,
       $writeable,
+      $fts,
       $create,
      ) = rearrange(['DSN',
 		    ['TEMP','TEMPORARY'],
@@ -238,6 +244,7 @@ sub init {
 		    ['PASS','PASSWD','PASSWORD'],
 		    ['OPTIONS','DBI_OPTIONS','DBI_ATTR'],
 		    ['WRITE','WRITEABLE'],
+		    'FTS',
 		    'CREATE',
 		   ],@_);
   $dbi_options  ||= {};
@@ -254,8 +261,13 @@ sub init {
     $dbh->do("PRAGMA synchronous = OFF;"); # makes writes much faster
     $dbh->do("PRAGMA temp_store = MEMORY;"); # less disk I/O; some speedup
     $dbh->do("PRAGMA cache_size = 20000;"); # less disk I/O; some speedup
+    # Keep track of database file location
+    my $cwd = getcwd;
+    my ($db_file) = ($dsn =~ m/(?:db(?:name)?|database)=(.+)$/);
+    $self->{dbh_file} = "$cwd/$db_file";
   }
   $self->{dbh}       = $dbh;
+  $self->{fts}       = $fts;
   $self->{is_temp}   = $is_temporary;
   $self->{namespace} = $namespace;
   $self->{writeable} = $writeable;
@@ -295,14 +307,14 @@ END
 	  typelist => <<END,
 (
   id  integer primary key autoincrement,
-  tag text    not null
+  tag text    not null collate nocase
 );
 create index index_typelist on typelist (tag);
 END
 	  name => <<END,
 (
   id           integer not null,
-  name         text    not null,
+  name         text    not null collate nocase,
   display_name integer default 0
 );
 create index index_name_id on name(id);
@@ -313,7 +325,7 @@ END
 (
   id              integer not null,
   attribute_id    integer not null,
-  attribute_value text
+  attribute_value text collate nocase
 );
 create index index_attribute_id    on attribute(attribute_id);
 create index index_attribute_value on attribute(attribute_value);
@@ -324,15 +336,14 @@ END
   id  integer primary key autoincrement,
   tag text    not null
 );
-create index index_attributelist_id  on attributelist(id);
 create index index_attributelist_tag on attributelist(tag);
 END
 	  parent2child => <<END,
 (
-  id    integer not null,
-  child integer not null
-);
-create unique index index_parent2child_id_child on parent2child(id,child);
+  id    integer,
+  child integer,
+  primary key(id, child)
+) without rowid;
 END
 
 	  meta => <<END,
@@ -351,6 +362,10 @@ END
 END
 	 };
 
+  if ($self->{'fts'}) {
+    delete($defs->{attribute});
+  }
+
   unless ($self->_has_spatial_index) {
     $defs->{feature_location} = <<END;
 (
@@ -372,8 +387,8 @@ END
    seqid             integer not null,
    bin               integer not null,
    cum_count         integer not null,
-   unique(typeid,seqid,bin)
-);
+   primary key (typeid,seqid,bin)
+) without rowid;
 END
   }
   return $defs;
@@ -384,13 +399,16 @@ sub _init_database {
 
     # must do this first before calling table_definitions
     $self->_create_spatial_index;
+    $self->_create_attribute_fts;
     $self->SUPER::_init_database(@_);
 }
 
+# FIXME: ensure this works with _create_attribute_fts...
 sub init_tmp_database {
     my $self = shift;
     my $erase = shift;
     $self->_create_spatial_index;
+    $self->_create_attribute_fts;
     $self->SUPER::init_tmp_database(@_);
 }
 
@@ -404,6 +422,42 @@ sub _create_spatial_index{
     }
 }
 
+sub _create_attribute_fts{
+    my $self = shift;
+    my $dbh   = $self->dbh;
+    if ($self->{'fts'}) {
+        my @fts_versions;
+        for (@fts_versions = grep(/^ENABLE_FTS[0-9]+$/, DBD::SQLite::compile_options)) { s/ENABLE_// }
+        # use the latest supported FTS version.
+        # DBD::SQLite::compile_options appears to be sorted
+        # alphabetically, so this should work through version FTS9.
+        die 'fts not supported by this version of DBD::SQLite' if (!@fts_versions);
+        $dbh->do("DROP TABLE IF EXISTS attribute");
+        $dbh->do("CREATE VIRTUAL TABLE "
+             . $self->_attribute_table
+             . " USING " . $fts_versions[-1]
+             . "(id integer not null, attribute_id integer not null, attribute_value text)");
+    }
+}
+
+###
+# return 1 if an existing attribute table in the connected database is an FTS
+# table, else 0
+#
+sub _has_fts {
+    my $self = shift;
+    if (!defined($self->{'has_fts'})) {
+        # If the attribute table is a virtual table, assume it is an FTS
+        # table. Per http://www.sqlite.org/fileformat2.html:
+        # For (sqlite_master) rows that define views, triggers, and virtual
+        # tables, the rootpage column is 0 or NULL.
+        ($self->{'has_fts'}) = $self->dbh->selectrow_array("select count(*) from sqlite_master where type = 'table' and name = '"
+                                                           . $self->_attribute_table 
+                                                           . "' and (rootpage = 0 or rootpage is null);");
+    }
+    return $self->{'has_fts'};
+}
+
 sub _has_spatial_index {
     my $self = shift;
     return $self->{'_has_spatial_index'} if exists $self->{'_has_spatial_index'};
@@ -423,7 +477,8 @@ sub _finish_bulk_update {
     my $fh = $self->dump_filehandle($table);
     my $path = $self->dump_path($table);
     $fh->close;
-    open($fh, $path);
+
+    open $fh, '<', $path or $self->throw("Could not read file '$path': $!");
     my $qualified_table = $self->_qualify($table);
 
     my $sth;
@@ -766,7 +821,7 @@ sub _name_sql {
   my $from  = "$name_table as n";
   my ($match,$string) = $self->_match_sql($name);
 
-  my $where = "n.id=$join AND lower(n.name) $match";
+  my $where = "n.id=$join AND n.name $match COLLATE NOCASE";
   $where   .= " AND n.display_name>0" unless $allow_aliases;
   return ($from,$where,'',$string);
 }
@@ -781,30 +836,38 @@ sub _search_attributes {
   my $attributelist_table = $self->_attributelist_table;
   my $type_table          = $self->_type_table;
   my $typelist_table      = $self->_typelist_table;
+  my $has_fts             = $self->_has_fts;
 
   my @tags    = @$attribute_names;
   my $tag_sql = join ' OR ',("al.tag=?") x @tags;
 
   my $perl_regexp = join '|',@words;
 
-  my @wild_card_words = map { "%$_%" } @words;
-  my $sql_regexp = join ' OR ',("a.attribute_value LIKE ?")  x @words;
-  # CROSS JOIN disables SQLite's table reordering optimization
+  my $sql_regexp;
+  my @wild_card_words;
+  if ($has_fts) {
+      $sql_regexp = "a.attribute_value MATCH ?";
+      @wild_card_words = join(' OR ', @words);
+  } else {
+      $sql_regexp = join ' OR ',("a.attribute_value LIKE ?")  x @words;
+      @wild_card_words = map { "%$_%" } @words;
+  }
+  # CROSS JOIN hinders performance with FTS attribute table for DBD::SQLite 1.42
   my $sql = <<END;
 SELECT name,attribute_value,tl.tag,n.id
   FROM $attributelist_table        AS al
-       CROSS JOIN $attribute_table AS a  ON al.id = a.attribute_id
-       CROSS JOIN $name_table      AS n  ON n.id = a.id
-       CROSS JOIN $type_table      AS t  ON t.id = n.id
-       CROSS JOIN $typelist_table  AS tl ON tl.id = t.typeid
+       JOIN $attribute_table AS a  ON al.id = a.attribute_id
+       JOIN $name_table      AS n  ON n.id  = a.id
+       JOIN $type_table      AS t  ON t.id  = n.id
+       JOIN $typelist_table  AS tl ON tl.id = t.typeid
   WHERE ($tag_sql)
     AND ($sql_regexp)
     AND n.display_name=1
 END
   $sql .= "LIMIT $limit" if defined $limit;
-  $self->_print_query($sql,@tags,@words) if DEBUG || $self->debug;
+  $self->_print_query($sql,@tags,@wild_card_words) if DEBUG || $self->debug;
   my $sth = $self->_prepare($sql);
-  $sth->execute(@tags,@wild_card_words) or $self->throw($sth->errstr);
+  $sth->execute(@tags, @wild_card_words) or $self->throw($sth->errstr);
 
   my @results;
   while (my($name,$value,$type,$id) = $sth->fetchrow_array) {
@@ -830,8 +893,8 @@ sub _match_sql {
     $match = "LIKE ?";
     $string  = $name;
   } else {
-    $match = "= lower(?)";
-    $string  = lc($name);
+    $match = "= ? COLLATE NOCASE";
+    $string  = $name;
   }
   return ($match,$string);
 }
@@ -846,11 +909,14 @@ sub _attributes_sql {
   my $attribute_table       = $self->_attribute_table;
   my $attributelist_table   = $self->_attributelist_table;
 
-  my $from = "$attribute_table AS a INDEXED BY index_attribute_id, $attributelist_table AS al";
+  my $from = "$attribute_table AS a" . ($self->_has_fts 
+                                        ? ''
+                                        : " INDEXED BY index_attribute_id") . ", $attributelist_table AS al";
+  my $a_al_join = $self->_has_fts ? 'a.attribute_id MATCH al.id' : 'a.attribute_id=al.id';
 
   my $where = <<END;
   a.id=$join
-  AND   a.attribute_id=al.id
+  AND   $a_al_join
   AND ($wf)
 END
 
@@ -883,7 +949,7 @@ sub _types_sql {
     }
 
     if (length $source_tag) {
-      push @matches,"lower(tl.tag)=lower(?)";
+      push @matches,"tl.tag=? COLLATE NOCASE";
       push @args,"$primary_tag:$source_tag";
     } else {
       push @matches,"tl.tag LIKE ?";
@@ -1054,7 +1120,7 @@ sub _genericid {
   my ($table,$namefield,$name,$add_if_missing) = @_;
   my $qualified_table = $self->_qualify($table);
   my $sth = $self->_prepare(<<END);
-SELECT id FROM $qualified_table WHERE lower($namefield)=lower(?)
+SELECT id FROM $qualified_table WHERE $namefield=? COLLATE NOCASE
 END
   $sth->execute($name) or die $sth->errstr;
   my ($id) = $sth->fetchrow_array;
@@ -1113,8 +1179,8 @@ sub _dump_update_name_index {
   my $dbh     = $self->dbh;
   my ($names,$aliases) = $self->feature_names($obj);
   # unlike DBI::mysql, don't quote, as quotes will be quoted when loaded
-  print $fh join("\t",$id,lc($_),1),"\n" foreach @$names;
-  print $fh join("\t",$id,lc($_),0),"\n" foreach @$aliases;
+  print $fh join("\t",$id,$_,1),"\n" foreach @$names;
+  print $fh join("\t",$id,$_,0),"\n" foreach @$aliases;
 }
 
 sub _update_name_index {
@@ -1128,8 +1194,8 @@ sub _update_name_index {
 
   my $sth = $self->_prepare("INSERT INTO $name (id,name,display_name) VALUES (?,?,?)");
 
-  $sth->execute($id,lc $_,1) or $self->throw($sth->errstr)   foreach @$names;
-  $sth->execute($id,lc $_,0) or $self->throw($sth->errstr) foreach @$aliases;
+  $sth->execute($id,$_,1) or $self->throw($sth->errstr)   foreach @$names;
+  $sth->execute($id,$_,0) or $self->throw($sth->errstr) foreach @$aliases;
   $sth->finish;
 }
 
@@ -1196,6 +1262,16 @@ sub _dump_update_location_index {
     print $fh join("\t",@args),"\n";
 }
 
+sub DESTROY {
+    my $self = shift;
+    # Remove filehandles, so temporal files can be properly deleted
+    if (%DBI::installed_drh) {
+	DBI->disconnect_all;
+	%DBI::installed_drh = ();
+    }
+    undef $self->{dbh};
+}
+
 1;
 
 =head1 AUTHOR
