@@ -40,7 +40,7 @@ use File::Temp 'tempdir';
 use File::Spec;
 use Fcntl qw(O_CREAT O_RDWR);
 
-our $VERSION = '1.10';
+our $VERSION = '1.11';
 
 my %DBHandles;
 
@@ -59,6 +59,13 @@ sub new {
 
 sub DESTROY {
     my $self = shift;
+    # Destroy all filehandle references
+    # before trying to delete files and folder
+    %DBHandles = ();
+    undef $self->{IndexIt};
+    undef $self->{TopLevel};
+    undef $self->{Local2Global};
+    undef $self->{Parent2Child};
     rmtree $self->{tmppath};
 #    File::Temp::cleanup() unless $self->{keep};
 }
