@@ -81,7 +81,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution. Bug reports can be submitted via
 the web:
 
-  https://redmine.open-bio.org/projects/bioperl/
+  https://github.com/bioperl/bioperl-live/issues
 
 =head1 AUTHOR - Jason Stajich
 
@@ -105,8 +105,8 @@ use vars qw($EntrezLocation $UrlParamSeparatorValue %EntrezParams
             $EntrezGet $EntrezSummary $EntrezFetch %SequenceParams
             $XMLTWIG $DATA_CACHE $RELATIONS);
 use strict;
-
 use Bio::Taxon;
+use Bio::DB::NCBIHelper;
 
 eval {
     require XML::Twig;
@@ -118,7 +118,7 @@ if( $@ ) {
 
 use base qw(Bio::WebAgent Bio::DB::Taxonomy);
 
-$EntrezLocation = 'http://www.ncbi.nih.gov/entrez/eutils/';
+$EntrezLocation = $Bio::DB::NCBIHelper::HOSTBASE . '/entrez/eutils/';
 $EntrezGet      = 'esearch.fcgi';
 $EntrezFetch    = 'efetch.fcgi';
 $EntrezSummary  = 'esummary.fcgi';
@@ -195,7 +195,7 @@ sub _initialize {
 sub get_num_taxa {
     my ($self) = @_;
     # Use this URL query to get the ID of all the taxa in the NCBI Taxonomy database:
-    # http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=taxonomy&term=all[Filter]
+    # https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=taxonomy&term=all[Filter]
     # Only the first 20 taxa IDs are returned (good because the list is long),
     # and the total number is reported as well (which is what we are interested
     # in).
@@ -472,7 +472,7 @@ sub get_taxonids {
         $DATA_CACHE->{name_to_id}->{$query} = [@data];
     }
     
-    wantarray() ? @data : shift @data;
+    return wantarray() ? @data : shift @data;
 }
 
 *get_taxonid = \&get_taxonids;
