@@ -54,7 +54,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.  Bug reports can be submitted via the
 web:
 
-  https://redmine.open-bio.org/projects/bioperl/
+  https://github.com/bioperl/bioperl-live/issues
 
 =head1 AUTHORS - Ewan Birney & Lincoln Stein
 
@@ -123,7 +123,9 @@ sub next_seq {
     local $/ = "\n>";
     return unless my $entry = $self->_readline;
 
-    chomp($entry);
+    # Replacing chomp for s///, since chomp is not working in some cases
+    $entry =~ s/\n$//;
+    $entry =~ s/\r$//;
     if ($entry =~ m/\A\s*\Z/s)  { # very first one
         return unless $entry = $self->_readline;
         chomp($entry);
