@@ -66,7 +66,7 @@ Bio::SeqUtils - Additional methods for PrimarySeq objects
       -flip => 1 
     );
 
-    # delete a segment of a seqence (from pos 1000 to 1100, inclusive), 
+    # delete a segment of a sequence (from pos 1000 to 1100, inclusive),
     # again preserving features and annotations
     my $new_molecule = Bio::SeqUtils->cut( $seq, 1000, 1100 );
 
@@ -165,7 +165,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.  Bug reports can be submitted via the
 web:
 
-  https://redmine.open-bio.org/projects/bioperl/
+  https://github.com/bioperl/bioperl-live/issues
 
 =head1 AUTHOR - Heikki Lehvaslaiho
 
@@ -593,6 +593,11 @@ sub trunc_with_features {
     my $truncrange =
       Bio::Range->new( -start => $start, -end => $end, -strand => 0 );
 
+    # make sure that there is no annotation or features in $trunc
+    # (->trunc() now clone objects except for Bio::Seq::LargePrimarySeq)
+    $trunc->annotation->remove_Annotations;
+    $trunc->remove_SeqFeatures;
+
     # move annotations
     foreach my $key ( $seq->annotation->get_all_annotation_keys() ) {
         foreach my $value ( $seq->annotation->get_Annotations($key) ) {
@@ -1470,6 +1475,11 @@ sub revcom_with_features {
       unless $seq->isa('Bio::SeqI');
     my $revcom = $seq->revcom;
 
+    # make sure that there is no annotation or features in $trunc
+    # (->revcom() now clone objects except for Bio::Seq::LargePrimarySeq)
+    $revcom->annotation->remove_Annotations;
+    $revcom->remove_SeqFeatures;
+
     #move annotations
     foreach my $key ( $seq->annotation->get_all_annotation_keys() ) {
         foreach my $value ( $seq->annotation->get_Annotations($key) ) {
